package wsrf.counter.impl;

import java.util.Calendar;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.SimpleResourceProperty;

import wsrf.counter.LastIncrementType;

public class CounterResource implements Resource, ResourceProperties {
    /** The counter service WSDL namespace */
    public static final String COUNTER_NS =
        "http://course.example/Counter";
    /** The qualified name of the counter resource property document */
    public static final QName COUNTER_RP_DOC = new QName(
        COUNTER_NS, "CounterRPDoc"
    );
    /** The qualified name of the value resource property */    
    public static final QName VALUE_RP = new QName(
        COUNTER_NS, "ValueRP"
    );
    /** The qualified name of the History resource property */    
    public static final QName HISTORY_RP = new QName(
        COUNTER_NS, "HistoryRP"
    );
    /** The qualified name of the LastIncrement resource property */    
    public static final QName LAST_INCREMENT_RP = new QName(
        COUNTER_NS, "LastIncrementRP"
    );        
    
    private static Log logger = LogFactory.getLog(
        CounterResource.class.getName()
    );

    private ResourcePropertySet rpDoc = null;
    private ResourceProperty valueRP = null;
    private ResourceProperty historyRP = null;
    private ResourceProperty lastIncrementRP = null;        

    public void increment(int incr) {
	logger.debug("Adding " + incr + " ... ");

        // Update value resource property
        Integer value = (Integer) this.valueRP.get(0);
        this.valueRP.set(0, value.intValue() + incr);

        // Update history resource property
        this.historyRP.add(new Integer(incr));

        // Update LastIncrement resource property
        this.lastIncrementRP.set(
            0, new LastIncrementType(incr, Calendar.getInstance())
        );        
    }

    public void initialize() throws Exception {
        // Create the resource property document for this resource
        this.rpDoc = new SimpleResourcePropertySet(COUNTER_RP_DOC);

        // Create and initialize the value resource property
        this.valueRP = new SimpleResourceProperty(VALUE_RP);
        this.valueRP.add(new Integer(0));

        // Create the history resource property        
        this.historyRP = new SimpleResourceProperty(HISTORY_RP);        

        // Create and initialize the LastIncrement resource property        
        this.lastIncrementRP = new SimpleResourceProperty(LAST_INCREMENT_RP);
        this.lastIncrementRP.add(
            new LastIncrementType(0, Calendar.getInstance())
        );
        
        // Add resource properties to resource property document
        this.rpDoc.add(this.valueRP);
        this.rpDoc.add(this.historyRP);
        this.rpDoc.add(this.lastIncrementRP);                        
    }   

    public ResourcePropertySet getResourcePropertySet() {
        return this.rpDoc;
    }
}
